#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h> 
#include <avr/interrupt.h>
#include <stdio.h>


static volatile uint8_t flag = 0;

static volatile uint8_t count = 0;

static volatile uint8_t saved_tcnt0 = 0;
static volatile uint8_t saved_count = 0;

static volatile unsigned int aux = 0;

ISR(TIMER0_OVF_vect, ISR_NOBLOCK) // Other interrupts aren't blocked during this routine
{
    count++;
}

ISR(PCINT0_vect, ISR_NOBLOCK)
{
        if ((PINA & (1 << PA1)) != 0) {
        /* Echo rising edge */
            TCNT0 = 0;  
            count = 0;
        }
        else {
        /* Echo falling edge */
            saved_tcnt0 = TCNT0;
            saved_count = count;
            flag = 1;
        }
}


void HC_SR04_init()
{
    TCCR0A = 0;
    TCCR0B = 0;
    TIMSK0 = 0;

    sei(); // Enable global interrupts
    TCCR0B |= (1 << CS02); // Prescaler = 256 => T = 16us
    TIMSK0 |= (1 << TOIE0); // Enable Overflow interrupt

    /* Configure pins for trigger and echo */
    DDRA |= (1 << PA0); // Trigger
    DDRA &= ~(1 << PA1); // Echo

    PORTA &= ~(1 << PA0);

    /* Enable pin change interrupt on echo pin */
    PCICR |= (1 << PCIE0);
    PCMSK0 = 0;
    PCMSK0 |= (1 << PCINT1);

    PCMSK0 |= (1 << PCINT3);
}

float HC_SR04_get_distance()
{
    double sum = 0;
    flag = 0;
    count = 0;
    saved_tcnt0 = 0;
    saved_count = 0;
    
    /* Reset Trigger signal for a clean rise */
    PORTA &= ~(1 << PA0);
    _delay_us(5);

    /* Send Trigger signal for > 10us */
    PORTA |= (1 << PA0);
    _delay_us(11);
    PORTA &= ~(1 << PA0);

    // _delay_ms(5);
    while(flag == 0 && count < 5);

    sum = (((int)saved_tcnt0 + 255 * saved_count) * 16.0) * 0.017;
    /* Wait for the Echo */
    
    return sum;
}

int main() {
    HC_SR04_init();
    DDRD |= (1 << PD7);
    DDRD |= (1 << PD4);

    /* Set pb2 as input */
    DDRB &= ~(1 << PB2);
    /* Activate pull-up resistance for pin PB2 */
    PORTB |= (1 << PB2);

    while(1) {
        float dist = (int) HC_SR04_get_distance();
        _delay_ms(10);

        /* Do nothing */
        if(dist > 50)
            PORTD &= ~(1 << PD4);

        /* Start vibrating periodically */
        if(dist > 20 && dist <= 50) {
            PORTD &= ~(1 << PD4);
            aux = (int)dist;
            while(aux > 0) {
                _delay_ms(40);
                --aux;
            }

            PORTD |= (1 << PD4);
            aux = (int)dist;
            while(aux > 0) {
                _delay_ms(40);
                --aux;
            }
        }

        /* Vibrate continuously */
        if(dist <= 20)
            PORTD |= (1 << PD4);

        /* Check if PB2 is pressed and stop vibrating */
        if ((PINB & (1 << PB2)) == 0) {
            PORTD &= ~(1 << PD4);
            _delay_us(5000000);
        }
    }
    
    return 0;
}